function [sys,x0,str,ts] = sfun_onephase(t,x,u,flag,theta_phase,t_dead,t_min,type_signal)
%   Variable step S-function.
%   Simulation of one leg of the pwm-module, including non-linear
%   effects as blanking time and minimum turn-off time

switch flag,
   
  case 0,    % Initialization %
    [sys,x0,str,ts]=mdlInitializeSizes;
  
  case 3,    % Outputs %
    sys=mdlOutputs(t,x,u);

  case 4,    % GetTimeOfNextVarHit %
    sys=mdlGetTimeOfNextVarHit(t,x,u,theta_phase,t_dead,t_min,type_signal);  
  
  case 9,    % Terminate %
    sys=mdlTerminate(t,x,u); 
  
  case {1,2}    % Unhandled flags %
    sys = []; 
  
  otherwise    % Unexpected flags %
    error(['Unhandled flag = ',num2str(flag)]);

end

% end sfun_onephase

%
%=============================================================================
% mdlInitializeSizes
% Return the sizes, initial conditions, and sample times for the S-function.
%=============================================================================
%
function [sys,x0,str,ts]=mdlInitializeSizes

sizes = simsizes;

sizes.NumContStates  = 0;
sizes.NumDiscStates  = 0;
sizes.NumOutputs     = 5;    
sizes.NumInputs      = 4;
sizes.DirFeedthrough = 1;
sizes.NumSampleTimes = 1;     % at least one sample time is needed

sys = simsizes(sizes);

%
% initialize the initial conditions
%
x0  = [];
str = [];
k = -1;    			  % starting with -1 gives k=0 in the first step
dt_plot = 1.0e-8;   % sets the time between two following calculation steps
						  % needed to get high quality plots of the digital signals
%                     
% Initialize the UserData field of the block
%
set_param(gcbh,'UserData',[k,dt_plot,0     ,0     ,0       ,0        ,0       ,0     ,0       ]);
% vector is defined by:   [k,dt_plot,t_sync,t_next,t_a_u_on,t_a_u_off,t_a_l_on,t_a_l_off, u_st_a]
%
%
% initialize the array of sample times
%
ts  = [-2 0];      % variable sample time

% end mdlInitializeSizes

%
%=============================================================================
% mdlOutputs
% Return the block outputs.
%=============================================================================
%
function sys=mdlOutputs(t,x,u)

% the if-statement defines the s-function's digital output  
% dependent of the state of the counter vaiable k

variable_vect = get_param(gcbh,'UserData');

k = variable_vect(1);
t_sync = variable_vect(3);

% Denne taes bort ved  inkludere sys(5) i k=0,1,2,... osv.
sys(5) = variable_vect(9);

if k == 0
  sys(1) = 0;     % sample hit, output da_u is (still) equal to 0
  sys(2) = 1;     % -- '' --  , output da_l is (still) equal to 1
  sys(3) = t_sync;% -- '' --  , time for sync signal is set on the output
  sys(4) = 0;	   % -- '' --  , sync is set equal to 0
  %sys(5) = abs(-(4/u(3))*(t-round(t/u(3))*u(3)))-1;
  
elseif k == 1
  sys(1) = 0;     % time is equal to t_a_l_off-dt_plot, output da_u is (still) equal to 0
  sys(2) = 1;     % -------------- '' --------------  , output da_l is (still) equal to 1
  sys(3) = t_sync;% -------------- '' --------------  , t_sync is unchanched
  sys(4) = 0;	   % -------------- '' --------------  , sync is (still) equal to 0
  %sys(5) = abs(-(4/u(3))*(t-round(t/u(3))*u(3)))-1;
  
elseif k == 2    
  sys(1) = 0;     % time is equal to t_a_l_off, da_u is (still) equal to 0
  sys(2) = 0;     % ---------- '' ----------  , da_l is set equal to 0
  sys(3) = t_sync;% ---------- '' ----------  , t_sync is unchanched 
  sys(4) = 0;     % ---------- '' ----------  , sync is (still) equal to 0
  %sys(5) = abs(-(4/u(3))*(t-round(t/u(3))*u(3)))-1;
  
elseif k == 3    
  sys(1) = 0;     % time is equal to t_a_l_on-dt_plot, da_u is (still) equal to 0
  sys(2) = 0;     % ------------- '' --------------  , da_l is (still) equal to 0
  sys(3) = t_sync;% ------------- '' --------------  , t_sync is unchanched 
  sys(4) = 0;     % ------------- '' --------------  , sync is (still) equal to 0
  %sys(5) = abs(-(4/u(3))*(t-round(t/u(3))*u(3)))-1;
  
elseif k == 4
  sys(1) = 1;     % time is equal to t_a_u_on, da_u is set equal to 1
  sys(2) = 0;     % ---------- '' ---------- , da_l is (still) equal to 0
  sys(3) = t_sync;% ---------- '' ---------- , t_sync is unchanched
  sys(4) = 0;     % ---------- '' ---------- , sync is (still) equal to 0
  %sys(5) = abs(-(4/u(3))*(t-round(t/u(3))*u(3)))-1;
  
elseif k == 5
  sys(1) = 1;     % time is equal to t_sync-dt_plot, da_u is (still) equal to 1
  sys(2) = 0;     % ------------- '' ------------- , da_l is (still) equal to 0
  sys(3) = t_sync;% ------------- '' ------------- , t_sync is unchanched
  sys(4) = 0;     % ------------- '' ------------- , sync is (still) equal to 0
  %sys(5) = abs(-(4/u(3))*(t-round(t/u(3))*u(3)))-1;
  
elseif k == 6
  sys(1) = 1;     % time is equal to t_sync, da_u is (still) equal to 1
  sys(2) = 0;     % --------- '' --------- , da_l is (still) equal to 0
  sys(3) = t_sync;% --------- '' --------- , t_sync is unchanched
  sys(4) = 1;     % --------- '' --------- , sync is set equal to 1
  %sys(5) = abs(-(4/u(3))*(t-round(t/u(3))*u(3)))-1;
  
elseif k == 7
  sys(1) = 1;     % time is equal to t_a_u_off-dt_plot, da_u is (still) equal to 1
  sys(2) = 0;     % -------------- '' --------------- , da_l is (still) equal to 0
  sys(3) = t_sync;% -------------- '' --------------- , t_sync is unchanched
  sys(4) = 1;     % -------------- '' --------------- , sync is (still) equal to 1
  %sys(5) = abs(-(4/u(3))*(t-round(t/u(3))*u(3)))-1;
  
elseif k == 8      
  sys(1) = 0;     % time is equal to t_a_u_off, da_u is set equal to 0   
  sys(2) = 0;     % ---------- '' ----------  , da_l is (still) equal to 0
  sys(3) = t_sync;% ---------- '' ----------  , t_sync is unchanched
  sys(4) = 1;     % ---------- '' ----------  , sync is (still) equal to 1
  %sys(5) = abs(-(4/u(3))*(t-round(t/u(3))*u(3)))-1;
  
elseif k == 9      
  sys(1) = 0;     % time is equal to t_a_l_on-dt_plot, da_u is (still) equal to 0   
  sys(2) = 0;     % -------------- '' -------------  , da_l is (still) equal to 0
  sys(3) = t_sync;% -------------- '' -------------  , t_sync is unchanched
  sys(4) = 1;     % -------------- '' -------------  , sync is (still) equal to 1
  %sys(5) = abs(-(4/u(3))*(t-round(t/u(3))*u(3)))-1;
  
elseif k == 10        
  sys(1) = 0;     % time is equal to t_a_l_on, da_u is (still) equal to 0   
  sys(2) = 1;     % ---------- '' ---------- , da_l is set equal to 1
  sys(3) = t_sync;% ---------- '' ---------- , t_sync is unchanched
  sys(4) = 1;     % ---------- '' ---------- , sync is (still) equal to 1
  %sys(5) = abs(-(4/u(3))*(t-round(t/u(3))*u(3)))-1;
  
elseif k == 11        
  sys(1) = 0;     % time is equal to t_next-dt_plot, da_u is (still) equal to 0   
  sys(2) = 1;     % ------------- '' ------------- , da_l is (still) equal to 1
  sys(3) = t_sync;% ------------- '' ------------- , t_sync is unchanched
  sys(4) = 1;     % ------------- '' ------------- , sync is (still) equal to 1
  %sys(5) = abs(-(4/u(3))*(t-round(t/u(3))*u(3)))-1;
  
elseif k == 12        
  sys(1) = 0;     % enable is turned off, da_u is set equal to 0   
  sys(2) = 0;     % -------- '' ------- , da_l is set equal to 0
  sys(3) = t_sync;% -------- '' ------- , t_sync is set on the output
  sys(4) = 0;     % -------- '' ------- , sync is set equal to 0
  %sys(5) = 0;
  
end    % end if


% end mdlOutputs

%
%=============================================================================
% mdlGetTimeOfNextVarHit
% Return the time of the next hit for this block.  Note that the result is
% absolute time.
%=============================================================================
%
function sys=mdlGetTimeOfNextVarHit(t,x,u,theta_phase,t_dead,t_min,type_signal)

variable_vect = get_param(gcbh,'UserData');

U_st = u(1);
ksi_s = u(2);
T_tri = u(3); 
enable = u(4);

k = variable_vect(1);
dt_plot = variable_vect(2);
t_sync = variable_vect(3);
t_next = variable_vect(4);
t_a_u_on = variable_vect(5);
t_a_u_off = variable_vect(6);
t_a_l_on = variable_vect(7);
t_a_l_off = variable_vect(8);
u_st_a    =variable_vect(9);


% triangle
%u_tri=abs(-(4/T_tri)*(t-round(t/T_tri)*T_tri))-1;
%u_tri=abs(-(4/T_tri)*(t_sync-round(t_sync/T_tri)*T_tri))-1;

if (k == 11 | k ==12)
  k = 0;    % re-initalization of k
else
  k = k+1;  % counter variable adds one
end % end if

if k == 0
  if type_signal == 2
      u_st_a = U_st*(cos(ksi_s+theta_phase)-cos(3*ksi_s)/6);
  else    % type_signal == 1 or any other value 
     u_st_a = U_st*cos(ksi_s+theta_phase);                 % "default" 
  end % end if
  
  dt_a_u_on = 0.25*(1-u_st_a)*T_tri;
  if dt_a_u_on < t_min/2
    dt_a_u_on = t_min/2;
  elseif dt_a_u_on > T_tri/2-t_min/2
    dt_a_u_on = T_tri/2-t_min/2;
  end % end if   
     
  dt_a_u_off = T_tri-dt_a_u_on;
  t_a_u_on = t+dt_a_u_on+t_dead;    % t_dead is the blanking time
  t_a_u_off = t+dt_a_u_off;
  t_a_l_on = t+dt_a_u_off+t_dead;    % t_dead is the blanking time
  t_a_l_off = t+dt_a_u_on;
  t_sync = t+T_tri/2;
  t_next = t+T_tri;
  
  sys = t_a_l_off-dt_plot;  
 
  if enable == 0    % checking if enable is off
    sys = t_next;
    k = 12; 
  end % end if  
  
elseif k == 1   
  sys = t_a_l_off;
  
elseif k == 2   
  sys = t_a_u_on-dt_plot;
  
elseif k == 3   
  sys = t_a_u_on;
  
elseif k == 4  
  sys = t_sync-dt_plot;
  
elseif k == 5  
  sys = t_sync;
  
elseif k == 6  
  sys = t_a_u_off-dt_plot; 
  
elseif k == 7  
  sys = t_a_u_off; 
  
elseif k == 8  
  sys = t_a_l_on-dt_plot;
  
elseif k == 9  
  sys = t_a_l_on;
  
elseif k == 10  
  sys = t_next-dt_plot; 
      
elseif k == 11  
  sys = t_next; 
    
else    
  k = -1;    % re-initalization of k

end    % end if

set_param(gcbh,'UserData',[k,dt_plot,t_sync,t_next,t_a_u_on,t_a_u_off,t_a_l_on,t_a_l_off,u_st_a]);
% writes the variable values to the block's workspace memory

% end mdlGetTimeOfNextVarHit

%
%=============================================================================
% mdlTerminate
% Perform any end of simulation tasks.
%=============================================================================
%
function sys=mdlTerminate(t,x,u)

sys = [];

% end mdlTerminate
